local brain = require "brains/frogbrain"

local assets =
{
	Asset("ANIM", "anim/skd_boss.zip")
}

SetSharedLootTable( 'skd_boss',
{
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_food_other1',      1.00},
    {'skd_food_other1', 1.00},
    {"minotaurhorn",     1.00},
 
})

local chest_loot =
{
    {item = {"armorruins", "ruinshat", "ruins_bat"}, count = 1},
    {item = {"orangestaff", "yellowstaff"}, count = 1},
    {item = {"orangeamulet", "yellowamulet"}, count = 1},
    {item = {"yellowgem"}, count = {2, 4}},
    {item = {"orangegem"}, count = {2, 4}},
    {item = {"greengem"}, count = {2, 3}},
    {item = {"thulecite"}, count = {8, 14}},
    {item = {"thulecite_pieces"}, count = {12, 36}},
    {item = {"gears"}, count = {3, 6}},
}

local PHASE1 = 0.6

local function CalcSanityAura(inst, observer)
    return inst.components.sanityaura.aura
end

local RESTARGET_MUST_TAGS = {"player"}
local RETARGET_CANT_TAGS = {"skd_monster"}

local function retargetfn(inst)
    if not inst.components.health:IsDead() then
        return FindEntity(inst, 35, function(guy)
            if not guy.components.health:IsDead() then
                return guy.components.inventory ~= nil
            end
        end,
        RESTARGET_MUST_TAGS, 
        RETARGET_CANT_TAGS
        )
    end
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
end

local function OnHealthDelta(inst, data)
    if inst.components.health.currenthealth < 20000 then
        inst.level = 3
        inst.components.sanityaura.aura = -400 / 30

    elseif inst.components.health.currenthealth < 40000 then
        inst.level = 3
        inst.components.sanityaura.aura = -200 / 30

    else 
        inst.level = 1
        inst.components.sanityaura.aura = -100 / 30
    end    
end

local function OnRemoved(inst)
    local spawner = c_findnext("skd_boss_spawner") or nil
    if spawner ~= nil then
        spawner.components.timer:StartTimer("Spawn_Skd_Boss", 480 * 20)
    end
end

local function dospawnchest(inst, loading)
    local chest = SpawnPrefab("minotaurchest")
    local x, y, z = inst.Transform:GetWorldPosition()
    chest.Transform:SetPosition(x, 0, z)

    --Set up chest loot
    chest.components.container:GiveItem(SpawnPrefab("cave_regenerator"))

    local loot_keys = {}
    for i, _ in ipairs(chest_loot) do
        table.insert(loot_keys, i)
    end
    local max_loots = math.min(#chest_loot, chest.components.container.numslots - 1)
    loot_keys = PickSome(math.random(max_loots - 2, max_loots), loot_keys)

    for _, i in ipairs(loot_keys) do
        local loot = chest_loot[i]
        local item = SpawnPrefab(loot.item[math.random(#loot.item)])
        if item ~= nil then
            if type(loot.count) == "table" and item.components.stackable ~= nil then
                item.components.stackable:SetStackSize(math.random(loot.count[1], loot.count[2]))
            end
            chest.components.container:GiveItem(item)
        end
    end

    if not chest:IsAsleep() then
        chest.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn")

        local fx = SpawnPrefab("statue_transition_2")
        if fx ~= nil then
            fx.Transform:SetPosition(x, y, z)
            fx.Transform:SetScale(1, 2, 1)
        end

        fx = SpawnPrefab("statue_transition")
        if fx ~= nil then
            fx.Transform:SetPosition(x, y, z)
            fx.Transform:SetScale(1, 1.5, 1)
        end
    end

    if not loading then
        inst:Remove()
    end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
    inst.entity:AddMiniMapEntity()
    -- inst.entity:AddNetwork()

    inst.AnimState:SetBank("skd_boss")
    inst.AnimState:SetBuild("skd_boss")
    inst.AnimState:PlayAnimation("idle_stage1,2", true)

    inst.MiniMapEntity:SetIcon("skd_boss.tex")

    inst:AddTag("skd_monster")
    inst:AddTag("monster")
    inst:AddTag("scarytoprey")
    inst:AddTag("scarytooceanprey")

    MakeObstaclePhysics(inst, 3, 24)
    inst.DynamicShadow:SetSize(2.5, 1.5)
    inst.Transform:SetScale(2.5, 2.5, 2.5)

    -- inst.entity:SetPristine()

    -- if not TheWorld.ismastersim then
        -- return inst
    -- end

    inst.level = 1

    inst:AddComponent("inspectable")

    inst:AddComponent("timer")

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(60000)
    inst:ListenForEvent("healthdelta", OnHealthDelta)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(150)
    inst.components.combat:SetRange(30)
    inst.components.combat:SetAttackPeriod(2)
    inst.components.combat:SetRetargetFunction(3, retargetfn)

    local _getAttacked = inst.components.combat.GetAttacked
    inst.components.combat.GetAttacked = function(self, attacker, damage, weapon, stimuli)
        if damage and attacker and not attacker:HasTag("player") then
            damage = damage * 0.4 
        end            

        return _getAttacked(self, attacker, damage, weapon, stimuli) 
    end

    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aurafn = CalcSanityAura
    inst.components.sanityaura.aura = -100 / 30

    inst:AddComponent("locomotor") 
    inst.components.locomotor.walkspeed = 0
    inst.components.locomotor.runspeed = 0

    inst:AddComponent("knownlocations")

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable("skd_boss")

    --inst:AddComponent("sanityaura")
    --inst.components.sanityaura.aurafn = CalcSanityAura
    --inst.components.sanityaura.aura = -sanity_aura

    inst:SetBrain(brain)

    inst:SetStateGraph("SGskd_boss")
    inst:ListenForEvent("attacked", OnAttacked)
    inst:ListenForEvent("onremove", OnRemoved)
    inst:ListenForEvent("death", dospawnchest)

   --[[ inst.RemoveTask = inst:DoTaskInTime(0, function(inst)  --世界只能存在1只
        if TUNING.SKD_MONSTER_ATK ~= true then
            inst:Remove()
        end

        if inst.RemoveTask then
            inst.RemoveTask:Cancel()
            inst.RemoveTask = nil
        end    
    end)  ]]
    
    return inst
end

return Prefab("skd_boss", fn, assets)
