local assets =
{
    Asset("ANIM", "anim/skd_tentacle.zip")
}

local RETARGET_MUST_TAGS = { "_combat", "_health" }
local RETARGET_CANT_TAGS = { "prey", "skd_monster" }
local function retargetfn(inst)
    return FindEntity(
        inst,
        TUNING.TENTACLE_ATTACK_DIST,
        function(guy)
            return guy.prefab ~= inst.prefab
                and guy.entity:IsVisible()
                and not guy.components.health:IsDead()
                and (guy.components.combat.target == inst or
                    guy:HasTag("character") or
                    guy:HasTag("monster") or
                    guy:HasTag("animal"))
        end,
        RETARGET_MUST_TAGS,
        RETARGET_CANT_TAGS)
end

local function shouldKeepTarget(inst, target)
    return target ~= nil
        and target:IsValid()
        and target.entity:IsVisible()
        and target.components.health ~= nil
        and not target.components.health:IsDead()
        and target:IsNear(inst, TUNING.TENTACLE_STOPATTACK_DIST)
end

local function OnAttacked(inst, data)
    if data.attacker == nil then
        return
    end

    local current_target = inst.components.combat.target

    if current_target == nil then
        --Don't want to handle initiating attacks here;
        --We only want to handle switching targets.
        return
    elseif current_target == data.attacker then
        --Already targeting our attacker, just update the time
        inst._last_attacker = current_target
        inst._last_attacked_time = GetTime()
        return
    end

    local time = GetTime()
    if inst._last_attacker == current_target and
        inst._last_attacked_time + TUNING.TENTACLE_ATTACK_AGGRO_TIMEOUT >= time then
        --Our target attacked us recently, stay on it!
        return
    end

    --Switch to new target
    inst.components.combat:SetTarget(data.attacker)
    inst._last_attacker = data.attacker
    inst._last_attacked_time = time
end

local function OnDeath(inst, data)
    if inst.owner == nil then return end
    if inst.owner and inst.owner:IsValid() then
        inst.owner.components.health:DoDelta(-TUNING.TENTACLE_HEALTH * 2)
    end    
end

local function fn()
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddPhysics()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    -- inst.entity:AddNetwork()

    inst.Physics:SetCylinder(0.25, 2)

    inst:AddTag("skd_monster")
    inst:AddTag("skd_boss_tentacle")
    inst:AddTag("monster")
    inst:AddTag("tentacle")
    inst:AddTag("wet")
    inst:AddTag("WORM_DANGER")
    
    inst.AnimState:SetBank("tentacle")
    inst.AnimState:SetBuild("skd_tentacle")
    --inst.AnimState:PlayAnimation("atk_pre", false)
    --inst.AnimState:PlayAnimation("idle")
    inst.scrapbook_anim = "atk_idle"

 --[[   inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end]]

    inst._last_attacker = nil
    inst._last_attacked_time = nil

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(TUNING.TENTACLE_HEALTH)

    inst:AddComponent("combat")
    inst.components.combat:SetRange(TUNING.TENTACLE_ATTACK_DIST)
    inst.components.combat:SetDefaultDamage(TUNING.TENTACLE_DAMAGE)
    inst.components.combat:SetAttackPeriod(TUNING.TENTACLE_ATTACK_PERIOD)
    inst.components.combat:SetRetargetFunction(GetRandomWithVariance(2, 0.5), retargetfn)
    inst.components.combat:SetKeepTargetFunction(shouldKeepTarget)

    MakeLargeFreezableCharacter(inst)

    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED

    inst:AddComponent("inspectable")
    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('tentacle')

    inst:SetStateGraph("SGboss_tentacle")

    inst:ListenForEvent("attacked", OnAttacked)

    inst.owner = nil
    inst.persists = false

    inst:ListenForEvent("death", OnDeath)

    return inst
end

local function shadow_teleport_fn()   
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    -- inst.entity:AddNetwork()

    inst.AnimState:SetBank("shadow_teleport")
    inst.AnimState:SetBuild("shadow_teleport")
    inst.AnimState:PlayAnimation("portal_in", true)

    --inst.AnimState:SetDeltaTimeMultiplier(0.3)

    inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetSortOrder(3)    

    --inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")

    inst:AddTag("FX")
    inst:AddTag("NOCLICK")

   --[[ inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end]]

    inst:DoTaskInTime(4, inst.Remove)
    inst.persists = false

    return inst
end

return Prefab("skd_boss_tentacle", fn, assets),
       Prefab("skd_boss_shadow_fx", shadow_teleport_fn)