local brain = require "brains/frogbrain"
local brain2 = require "brains/skd_monster2_brain"

local function MakeMonster(name, health, damage, atk_range, speed, sanity_aura)
local assets =
{
	Asset("ANIM", "anim/"..name..".zip")
}

local sounds =
{
    pant = "dontstarve/creatures/hound/pant",
    attack = "dontstarve/creatures/hound/attack",
    bite = "dontstarve/creatures/hound/bite",
    bark = "dontstarve/creatures/hound/bark",
    death = "dontstarve/creatures/hound/death",
    sleep = "dontstarve/creatures/hound/sleep",
    growl = "dontstarve/creatures/hound/growl",
    howl = "dontstarve/creatures/together/clayhound/howl",
    hurt = "dontstarve/creatures/hound/hurt",
}

SetSharedLootTable( 'skd_monster1',
{
    {'skd_fishmeat',             1.00},
})

SetSharedLootTable( 'skd_monster2',
{
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_food_other1',             0.10},        
})

SetSharedLootTable( 'skd_monster3',
{
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_food_other1',             0.10},
    {'skd_food_other2',             0.05},       
})

SetSharedLootTable( 'skd_monster4',
{
    {'skd_fishmeat',             1.00},
    {'skd_fishmeat',             1.00},
    {'skd_food_other2',             0.05},        
})

local function CalcSanityAura(inst, observer)
    return inst.components.sanityaura.aura
end

local RESTARGET_MUST_TAGS = {"player"}
local RETARGET_CANT_TAGS = {"skd_monster"}

local function retargetfn(inst)
    if not inst.components.health:IsDead() and not inst.components.sleeper:IsAsleep() 
    and not (inst.prefab == "skd_monster2" and inst.components.health:GetPercent() >= 1) then
        return FindEntity(inst, 30, function(guy)
            if not guy.components.health:IsDead() then
                return guy.components.inventory ~= nil
            end
        end,
        RESTARGET_MUST_TAGS, 
        RETARGET_CANT_TAGS
        )
    end
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
end

local function DoAreaAtk(inst)
    if inst.components.health:IsDead() then return end

    local x, y, z = inst.Transform:GetWorldPosition()
    local exclude_tags = {'FX', 'NOCLICK', 'INLIMBO', 'playerghost'}
    local ents = TheSim:FindEntities(x, y, z, 8, nil, exclude_tags) 
    for k, v in ipairs(ents) do
        if v and v:IsValid() and (v:HasTag("player") or v.prefab == "skd_monster3") and not v.components.health:IsDead() then
            inst.flies = inst:SpawnChild("skd_monster2_fx")
            inst.flies.Transform:SetScale(1, 1, 1) 
            v.components.combat:GetAttacked(inst, damage)

            if v.components.sanity then
                 v.components.sanity:DoDelta(-(damage * 0.05))
            end    
        end    
    end     
end

local function HealthDelta(inst, data)
    if data and data.oldpercent and data.newpercent then
    if (data.oldpercent > 0.85 and data.newpercent <= 0.85) or (data.oldpercent > 0.7 and data.newpercent <= 0.7)
    or (data.oldpercent > 0.55 and data.newpercent <= 0.55)  or (data.oldpercent > 0.4 and data.newpercent <= 0.4) 
    or (data.oldpercent > 0.25 and data.newpercent <= 0.25)  or (data.oldpercent > 1 and data.newpercent <= 1) then
        DoAreaAtk(inst)
    end
    end    
end

local function DoAtk(inst)
    if inst.components.health:IsDead() then return end

    local x, y, z = inst.Transform:GetWorldPosition()
    local exclude_tags = {'FX', 'NOCLICK', 'INLIMBO', 'playerghost'}
    local ents = TheSim:FindEntities(x, y, z, 3, nil, exclude_tags) 
    for k, v in ipairs(ents) do
        if v and v:IsValid() and (v:HasTag("player") or v.prefab == "skd_monster3") and not v.components.health:IsDead() then
            inst.flies = inst:SpawnChild("skd_monster4_fx")
            inst.flies.Transform:SetScale(1, 1, 1) 

            v.components.combat:GetAttacked(inst, damage)

            if v.components.sanity then
                v.components.sanity:DoDelta(-(damage * 0.05))
            end    
        end    
    end     
end

local function Live(inst)
    inst:SetBrain(brain)
    inst:DoPeriodicTask(1, function()
        if inst.components.combat.target then
            inst.components.locomotor.walkspeed = 10
            inst.components.locomotor.runspeed = 10
        else
            inst.components.locomotor.walkspeed = speed
            inst.components.locomotor.runspeed = speed               
        end 

        inst.components.health:DoDelta(-(inst.components.health.maxhealth * 0.04))   
    end)
end

local function LoadHealthDelta(inst)
    if inst.components.health:GetPercent() < 1 then
        Live(inst) 
    end
end

local function OnHealthDelta(inst, data)
    if data.newpercent < 1 and data.oldpercent >= 1 then
        Live(inst)
    end    
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
    -- inst.entity:AddNetwork()

    inst.AnimState:SetBank(name == "skd_monster1" and "hound" or name)
    inst.AnimState:SetBuild(name)
    inst.AnimState:PlayAnimation("idle")

    inst:AddTag("skd_monster")
    inst:AddTag("monster")
    inst:AddTag("scarytoprey")
    inst:AddTag("scarytooceanprey")

    MakeCharacterPhysics(inst, 10, .5)
    
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.GROUND)  
    inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    -- inst.Physics:CollidesWith(COLLISION.SMALLOBSTACLES)

    inst.DynamicShadow:SetSize(2.5, 1.5)
    
    if name == "skd_monster1" then
       inst.Transform:SetFourFaced()

    elseif name == "skd_monster3" then     
       inst.Transform:SetTwoFaced()
       inst.Transform:SetScale(2, 2, 2)

    else
       inst.Transform:SetTwoFaced()
       inst.Transform:SetScale(1.7, 1.7, 1.7)       
    end

    -- inst.entity:SetPristine()

    -- if not TheWorld.ismastersim then
        -- return inst
    -- end

    inst.sounds = sounds

    inst:AddComponent("inspectable")

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(health)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(damage)
    inst.components.combat:SetRange(atk_range)
    inst.components.combat:SetAttackPeriod(2)
    inst.components.combat:SetRetargetFunction(3, retargetfn)

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = speed
    inst.components.locomotor.runspeed = speed

    inst:AddComponent("knownlocations")

    inst:AddComponent("follower")

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable(name)

    inst:AddComponent("sleeper")

    if sanity_aura then
    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aurafn = CalcSanityAura
    inst.components.sanityaura.aura = -sanity_aura
    end

    inst:SetBrain(brain)

    if name == "skd_monster1" then
        inst:SetStateGraph("SGhound")
    else
        inst:SetStateGraph("SGskd_monster")  
    end

    if name == "skd_monster2" then
        inst:SetBrain(brain2)

        inst.components.locomotor.walkspeed = 0.1
        inst.components.locomotor.runspeed = 0.2

        inst:DoTaskInTime(0, LoadHealthDelta)
        inst:ListenForEvent("healthdelta", OnHealthDelta)
        inst.flies = inst:SpawnChild("skd_monster2_fx")
    end     

    if name == "skd_monster3" then
        inst:ListenForEvent("healthdelta", HealthDelta)
    end   

    if name == "skd_monster4" then
        inst:DoPeriodicTask(1, DoAtk)
    end   

    inst:ListenForEvent("attacked", OnAttacked)
 --   inst:ListenForEvent("death", GetWorld():RemoveTag("spawn_skd_moster"))

    return inst
end

return Prefab(name, fn, assets)
end   

return MakeMonster("skd_monster1", 500, 46, 2, 10, 200/30),
       MakeMonster("skd_monster2", 3000, 67, 3, 2, 900/30),
       MakeMonster("skd_monster3", 4000, 300, 2, 3),
       MakeMonster("skd_monster4", 1500, 28, 2, 4)