GLOBAL.setmetatable(env,{__index=function(t,k) return GLOBAL.rawget(GLOBAL,k) end})

AddStategraphPostInit("wilson", function(sg)
    sg.states["skd_atk"] = State {
        name = "skd_atk",
        tags = { "attack", "notalking", "abouttoattack", "autopredict" },
        onenter = function(inst)
            if inst.components.combat:InCooldown() then
                inst.sg:RemoveStateTag("abouttoattack")
                inst:ClearBufferedAction()
                inst.sg:GoToState("idle", true)
                return
            end

            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("atk_prop_pre")
            inst.AnimState:PushAnimation("atk_prop", false)

            local buffaction = inst:GetBufferedAction()
            local target = buffaction ~= nil and buffaction.target or nil

            if buffaction then 
                local target = buffaction.target or nil
                inst.components.combat:SetTarget(target)
                inst.components.combat:StartAttack()
                inst.sg.statemem.target = target
            end
        end,

        timeline =
        {
            TimeEvent(8 * FRAMES, function(inst)--改攻速帧数
                inst:PerformBufferedAction()
                inst.components.combat:DoAttack(inst.sg.statemem.target)
                inst.sg:RemoveStateTag("abouttoattack")
            end),

            TimeEvent(16 * FRAMES, function(inst)--改攻速帧数
                inst.sg:GoToState("idle", true)
            end),
        },

        onexit = function(inst)
            inst.components.combat:SetTarget(nil)
            --[[if inst.sg:HasStateTag("abouttoattack") then
                inst.components.combat:CancelAttack()
            end]]
        end,

        events =
        {
            EventHandler("equip", function(inst) inst.sg:GoToState("idle") end),
            EventHandler("unequip", function(inst) inst.sg:GoToState("idle") end),
            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },
    }   
end)

AddStategraphPostInit("wilson_client", function(sg)
    sg.states["skd_atk"] = State {
        name = "skd_atk",
        tags = { "attack", "notalking", "abouttoattack" },
        server_states = { "skd_atk" },
        onenter = function(inst)
            local combat = inst.replica.combat
            if combat:InCooldown() then
                inst.sg:RemoveStateTag("abouttoattack")
                inst:ClearBufferedAction()
                inst.sg:GoToState("idle", true)
                return
            end

            combat:StartAttack()
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("atk_prop_pre") 
            inst.AnimState:PushAnimation("atk_prop", false)

            local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil then
                inst:PerformPreviewBufferedAction()

                if buffaction.target ~= nil and buffaction.target:IsValid() then
                    inst:FacePoint(buffaction.target:GetPosition())
                    inst.sg.statemem.attacktarget = buffaction.target
                    inst.sg.statemem.retarget = buffaction.target
                end
            end

            inst.sg:SetTimeout(16 * FRAMES)
        end,

        ontimeout = function(inst)
            inst:ClearBufferedAction()
            inst.sg:GoToState("idle")
        end,
    }
end)

AddStategraphState('wilson',
    State{
        name = "scared",
        tags = {"doing", "busy"}, 

        onenter = function(inst)
            inst.sg:SetTimeout(4)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("idle_shiver_pre", false)
            inst.AnimState:PushAnimation("idle_shiver_loop", true)

            if inst.components.playercontroller ~= nil then
                -- inst.components.playercontroller:RemotePausePrediction()
                -- inst.components.playercontroller:EnableMapControls(false)
                inst.components.playercontroller:Enable(false)
            end            
        end,
        
        ontimeout = function(inst)
            inst:PerformBufferedAction()
            inst.sg:GoToState("idle", true)
        end,

        onexit = function(inst)
            inst.sg:RemoveStateTag("busy")

            if inst.components.playercontroller ~= nil then
                -- inst.components.playercontroller:EnableMapControls(true)
                inst.components.playercontroller:Enable(true)
            end            
        end,       
    }
)

AddStategraphState('wilson_client',
    State{
        name = "scared",  
        tags = {"doing", "busy"}, 

        onenter = function(inst)
            inst.sg:SetTimeout(4)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("idle_shiver_pre", false)
            inst.AnimState:PushAnimation("idle_shiver_loop", true)
        end,
        
        ontimeout = function(inst)
            inst:ClearBufferedAction()
            inst.sg:GoToState("idle", true)
        end,

        onexit = function(inst)
            inst.sg:RemoveStateTag("busy")
            inst:ClearBufferedAction()
        end,               
    }
)

AddStategraphPostInit("wilson", function(sg)
local event_doattack = sg.events["doattack"]
	local event_doattack_oldfn = event_doattack.fn
	event_doattack.fn = function(inst, data)
		if not inst.components.health:IsDead() and not inst.sg:HasStateTag("attack") then
            local weapon = inst.components.combat and inst.components.combat:GetWeapon()
            if weapon and weapon:HasTag("skd_sword") then
				inst.sg:GoToState("skd_atk")
            else
                return event_doattack_oldfn(inst, data)
            end
        end
	end
end)

AddStategraphPostInit("wilson_client", function(sg)
local event_doattack = sg.events["doattack"]
	local event_doattack_oldfn = event_doattack.fn
	event_doattack.fn = function(inst, data)
		if not inst.components.health:IsDead() and not inst.sg:HasStateTag("attack") then
            local weapon = inst.components.combat and inst.components.combat:GetWeapon()
            if weapon and weapon:HasTag("skd_sword") then
				inst.sg:GoToState("skd_atk")
            else
                return event_doattack_oldfn(inst, data)
            end
        end
	end
end)