require("stategraphs/commonstates")

local function GetSpawnPoint(pos, radius)
    -- local x, y, z = inst.Transform:GetWorldPosition()
    local rad = radius or math.random(1, 20)
    local angle = math.random() * 2 * PI

    return pos.x + rad * math.cos(angle), pos.y, pos.z - rad * math.sin(angle)
end

local function SpawnRock(inst)
    for i = 1, math.random(6, 8) do
    local pos = inst:GetPosition()
    local x1, y1, z1 = GetSpawnPoint(pos)
    local rock = SpawnPrefab("skd_boss_shadowmeteor")
    rock.Transform:SetPosition(x1, y1, z1)
    end
end

local WAVE_SPAWN_DISTANCE = 1.5
local function SpawnAttackWaves(inst, position, rotation, spawn_radius, numWaves, totalAngle, waveSpeed, wavePrefab, idleTime, instantActive)  
    wavePrefab = wavePrefab or "skd_boss_wave"
    waveSpeed = waveSpeed or 6
    idleTime = idleTime or 5
    totalAngle = (numWaves == 1 and 0) or
            (totalAngle and (totalAngle % 361)) or
            360

    local anglePerWave = (totalAngle == 0 and 0) or
            (totalAngle == 360 and totalAngle/numWaves) or
            totalAngle/(numWaves - 1)

    local startAngle = rotation or math.random(-180, 180)
    local total_rad = (spawn_radius or 0.0) + WAVE_SPAWN_DISTANCE

    local wave_spawned = false
    for i = 0, numWaves - 1 do
        local angle = (startAngle - (totalAngle/2)) + (i * anglePerWave)
        local offset_direction = Vector3(math.cos(angle*DEGREES), 0, -math.sin(angle*DEGREES)):Normalize()
        local wavepos = position + (offset_direction * total_rad)

        --if not TheWorld.Map:IsPassableAtPoint(wavepos:Get()) then
            wave_spawned = true

            local wave = SpawnPrefab(wavePrefab)
            wave.owner = inst
            wave.Transform:SetPosition(wavepos:Get())
            wave.Transform:SetRotation(angle)
            if type(waveSpeed) == "table" then
                wave.Physics:SetMotorVel(waveSpeed[1], waveSpeed[2], waveSpeed[3])
            else
                wave.Physics:SetMotorVel(waveSpeed, 0, 0)
            end
            wave.idle_time = idleTime

            if instantActive then
                wave.sg:GoToState((idleTime > 0 and "instant_rise") or "lower")
            end
        --end
    end

    -- Let our caller know if we actually spawned at least 1 wave.
    return wave_spawned
end

local function spawnwaves(inst, numWaves, totalAngle, waveSpeed, wavePrefab, initialOffset, idleTime, instantActivate, random_angle)
    SpawnAttackWaves(
        inst,
        inst:GetPosition(),
        (not random_angle and inst.Transform:GetRotation()) or nil,
        initialOffset or (inst.Physics and inst.Physics:GetRadius()) or nil,
        numWaves,
        totalAngle,
        waveSpeed,
        wavePrefab,
        idleTime,
        instantActivate
    )
end

local function GetSpawnPoint(pos, radius)
    -- local x, y, z = inst.Transform:GetWorldPosition()
    local rad = radius or math.random(4, 5)
    local angle = math.random() * 2 * PI

    return pos.x + rad * math.cos(angle), pos.y, pos.z - rad * math.sin(angle)
end

local function SpawnTent(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    local exclude_tags = {'FX', 'NOCLICK', 'INLIMBO'}
    local ents = TheSim:FindEntities(x, 0, z, 12, { "skd_boss_tentacle" }, exclude_tags) 
    local get_ten = (#ents > 0 and #ents) or 0

    if get_ten >= 0 and get_ten < 8 then
    for i = 1, 2 do
    local pos = inst:GetPosition()
    local x1, y1, z1 = GetSpawnPoint(pos)
    local tent = SpawnPrefab("skd_boss_tentacle")
    tent.Transform:SetPosition(x1, y1, z1)

    tent.owner = inst
    end
    end
end

local function ControlPlayer(inst)
    local nottags = {'FX', 'NOCLICK', 'INLIMBO', 'playerghost'} 
    local x, y, z = inst.Transform:GetWorldPosition() 
    local ents = TheSim:FindEntities(x, y, z, 12, { "player" }, nottags) 
    for k, v in ipairs(ents) do
        if v.sg and not v.sg:HasStateTag("dead") and v.components.health and not v.components.health:IsDead() then
            v.sg:GoToState("scared")

            local lightningfx = SpawnPrefab("skd_boss_shadow_fx")
            lightningfx.Transform:SetPosition(v:GetPosition():Get())  
        end
    end
end

local events=
{
    EventHandler("death", function(inst) inst.sg:GoToState("death") end),
    EventHandler("doattack", function(inst) 
    local Skill1_Cd = inst.components.timer ~= nil and (inst.components.timer:GetTimeLeft("skill1_cd") or 0) or nil 
    local Skill2_Cd = inst.components.timer ~= nil and (inst.components.timer:GetTimeLeft("skill2_cd") or 0) or nil
    local Skill3_Cd = inst.components.timer ~= nil and (inst.components.timer:GetTimeLeft("skill3_cd") or 0) or nil 
    local Skill4_Cd = inst.components.timer ~= nil and (inst.components.timer:GetTimeLeft("skill4_cd") or 0) or nil    
    if not inst.components.health:IsDead() and not inst.sg:HasStateTag("busy") and not inst.sg:HasStateTag("attack") then
        if Skill1_Cd == nil or (Skill1_Cd and Skill1_Cd <= 0) then 
            inst.sg:GoToState("attack1")

        elseif Skill2_Cd == nil or (Skill2_Cd and Skill2_Cd <= 0) then
            inst.sg:GoToState("attack2")

        elseif Skill3_Cd == nil or (Skill3_Cd and Skill3_Cd <= 0) and inst.level >= 2 then
            inst.sg:GoToState("attack3")

        elseif Skill4_Cd == nil or (Skill4_Cd and Skill4_Cd <= 0) and inst.level > 2 then
            inst.sg:GoToState("attack4")                                            
        end
    end 
    end),
}

local states=
{

    State{
        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst, pushanim)
            inst.Physics:Stop() 
            inst.AnimState:PlayAnimation("idle_stage1,2", true)
        end,
    },

    State{
        name = "attack1",
        tags = {"attack"},

        onenter = function(inst, cd)
            inst.Physics:Stop()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation("skill_shuiquan_stage1,2")
        end,

        timeline=
        {
            TimeEvent(32*FRAMES, function(inst) --inst.components.combat:DoAttack() 
                SpawnRock(inst)
                local time = (inst.level == 1 and 25) or (inst.level == 2 and 20) or 15
                inst.components.timer:StartTimer("skill1_cd", time)
            end),
        },

        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end),
        },
    },


    State{
        name = "attack2",
        tags = {"attack"},

        onenter = function(inst, cd)
            inst.Physics:Stop()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation("skill_ice_stage2")
        end,

        timeline=
        {
            TimeEvent(85*FRAMES, function(inst) --inst.components.combat:DoAttack() 
                spawnwaves(inst, 12, 360, 4, nil, 2, 2, nil, true) 
                local time = (inst.level == 1 and 45) or (inst.level == 2 and 35) or 30
                inst.components.timer:StartTimer("skill2_cd", 10)
            end),
        },

        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end),
        },
    },

    State{
        name = "attack3",
        tags = {"attack"},

        onenter = function(inst, cd)
            inst.Physics:Stop()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation("skill_shuiquan_stage1,2")
        end,

        timeline=
        {
            TimeEvent(32*FRAMES, function(inst) --inst.components.combat:DoAttack() 
                SpawnTent(inst)
                inst.components.timer:StartTimer("skill3_cd", 40)
            end),
        },

        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end),
        },
    },

    State{
        name = "attack4",
        tags = {"attack"},

        onenter = function(inst)
            inst.Physics:Stop()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation("skill_jingshen3.0")
        end,

        timeline=
        {
            TimeEvent(30*FRAMES, function(inst) --inst.components.combat:DoAttack() 
                ControlPlayer(inst)
                inst.components.timer:StartTimer("skill4_cd", 40)
            end),
        },

        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end),  --c_findnext("skd_boss").sg:GoToState("attack3")
        },
    },


    State{
        name = "death",
        tags = {"busy"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("death")
            inst.Physics:Stop()
            RemovePhysicsColliders(inst)
            inst.components.lootdropper:DropLoot(inst:GetPosition())
        end,
    },
}

CommonStates.AddFrozenStates(states)
-- CommonStates.AddSinkAndWashAsoreStates(states)

return StateGraph("skd_boss", states, events, "idle")
